@@ -40,15 +40,15 @@ class Principal(object):
 
     @staticmethod
     def parse(obj):
-        realm = obj.get(u'realm', None)
-        authid = obj.get(u'authid', None)
-        role = obj.get(u'role', None)
-        extra = obj.get(u'extra', None)
+        realm = obj.get('realm', None)
+        authid = obj.get('authid', None)
+        role = obj.get('role', None)
+        extra = obj.get('extra', None)
         return Principal(realm, authid, role, extra)
 
 
 class Node(object):
-    def __init__(self, node_id, heartbeat, heartbeat_time, status=u'online'):
+    def __init__(self, node_id, heartbeat, heartbeat_time, status='online'):
         self.node_id = node_id
         self.heartbeat = heartbeat
         self.heartbeat_time = heartbeat_time
@@ -56,10 +56,10 @@ class Node(object):
 
     def marshal(self):
         return {
-            u'id': self.node_id,
-            u'heartbeat': self.heartbeat,
-            u'timestamp': self.heartbeat_time,
-            u'status': self.status,
+            'id': self.node_id,
+            'heartbeat': self.heartbeat,
+            'timestamp': self.heartbeat_time,
+            'status': self.status,
         }
 
     @staticmethod
@@ -83,59 +83,59 @@ class Authenticator(ApplicationSession):
     frontend WAMP connections to CFC, for both CF nodes, CFC UI and user CFC scripts.
     """
 
-    GLOBAL_USER_REALM = u'com.crossbario.fabric'
+    GLOBAL_USER_REALM = 'com.crossbario.fabric'
     """
     Global users realm on Crossbar.io.
     """
 
-    GLOBAL_USER_REALM_USER_ROLE = u'user'
+    GLOBAL_USER_REALM_USER_ROLE = 'user'
     """
     The WAMP authrole regular users get on the Crossbar.io domain (global) users
     realm. A role different from this only makes sense for Crossbar.io admins.
     """
 
-    MREALM_USER_ROLES = [u'guest', u'developer', u'operator', u'admin', u'owner']
+    MREALM_USER_ROLES = ['guest', 'developer', 'operator', 'admin', 'owner']
     """
     All permissible roles a user can take on a management realm. This is a fixed
     set hardwired into Crossbar.io!
     """
 
-    MREALM_CREATOR_DEFAULT_ROLES = [u'owner']
+    MREALM_CREATOR_DEFAULT_ROLES = ['owner']
     """
     THe set of roles a user creating a new management realm gets by default.
     """
 
-    MREALM_NODE_ROLE = u'node'
+    MREALM_NODE_ROLE = 'node'
     """
     The (fixed) role a user node gets when joining the management realm it is
     paired to.
     """
 
-    ERROR_AUTH_INVALID_PARAMETERS = u'fabric.auth-failed.invalid-parameters'
-    ERROR_AUTH_INVALID_PARAMETERS_MSG = u'Invalid parameters in authentication: {}'
+    ERROR_AUTH_INVALID_PARAMETERS = 'fabric.auth-failed.invalid-parameters'
+    ERROR_AUTH_INVALID_PARAMETERS_MSG = 'Invalid parameters in authentication: {}'
 
-    ERROR_AUTH_PENDING_ACT = u'fabric.auth-failed.pending-activation'
-    ERROR_AUTH_PENDING_ACT_MSG = u'There is a pending activation (from {} ago) - please check your email inbox, or request a new code'
+    ERROR_AUTH_PENDING_ACT = 'fabric.auth-failed.pending-activation'
+    ERROR_AUTH_PENDING_ACT_MSG = 'There is a pending activation (from {} ago) - please check your email inbox, or request a new code'
 
-    ERROR_AUTH_NO_PENDING_ACT = u'fabric.auth-failed.no-pending-activation'
-    ERROR_AUTH_NO_PENDING_ACT_MSG = u'There is no (pending) activation for this user/pubkey, but an activation code was provided'
+    ERROR_AUTH_NO_PENDING_ACT = 'fabric.auth-failed.no-pending-activation'
+    ERROR_AUTH_NO_PENDING_ACT_MSG = 'There is no (pending) activation for this user/pubkey, but an activation code was provided'
 
-    ERROR_AUTH_INVALID_ACT_CODE = u'fabric.auth-failed.invalid-activation-code'
-    ERROR_AUTH_INVALID_ACT_CODE_MSG = u'This activation code is invalid: {}'
+    ERROR_AUTH_INVALID_ACT_CODE = 'fabric.auth-failed.invalid-activation-code'
+    ERROR_AUTH_INVALID_ACT_CODE_MSG = 'This activation code is invalid: {}'
 
-    ERROR_AUTH_NODE_UNPAIRED = u'fabric.auth-failed.node-unpaired'
-    ERROR_AUTH_NODE_UNPAIRED_MSG = u'This node is unpaired. Please pair the node with management realm first.'
+    ERROR_AUTH_NODE_UNPAIRED = 'fabric.auth-failed.node-unpaired'
+    ERROR_AUTH_NODE_UNPAIRED_MSG = 'This node is unpaired. Please pair the node with management realm first.'
 
-    ERROR_AUTH_NODE_ALREADY_CONNECTED = u'fabric.auth-failed.node-already-connected'
-    ERROR_AUTH_NODE_ALREADY_CONNECTED_MSG = u'A node with this pubkey/node_id/authid is already connected.'
+    ERROR_AUTH_NODE_ALREADY_CONNECTED = 'fabric.auth-failed.node-already-connected'
+    ERROR_AUTH_NODE_ALREADY_CONNECTED_MSG = 'A node with this pubkey/node_id/authid is already connected.'
 
-    ERROR_AUTH_EMAIL_FAILURE = u'fabric.auth-failed.email-failure'
+    ERROR_AUTH_EMAIL_FAILURE = 'fabric.auth-failed.email-failure'
 
-    ERROR_AUTH_NEW_USER = u'fabric.auth-failed.new-user-auth-code-sent'
-    ERROR_AUTH_NEW_USER_MSG = u'We have sent an authentication code to {email}.'
+    ERROR_AUTH_NEW_USER = 'fabric.auth-failed.new-user-auth-code-sent'
+    ERROR_AUTH_NEW_USER_MSG = 'We have sent an authentication code to {email}.'
 
-    ERROR_AUTH_REGISTERED_USER = u'fabric.auth-failed.registered-user-auth-code-sent'
-    ERROR_AUTH_REGISTERED_USER_MSG = u'We have sent an authentication code to {email}.'
+    ERROR_AUTH_REGISTERED_USER = 'fabric.auth-failed.registered-user-auth-code-sent'
+    ERROR_AUTH_REGISTERED_USER_MSG = 'We have sent an authentication code to {email}.'
 
     def __init__(self, config):
         ApplicationSession.__init__(self, config)
@@ -178,7 +178,7 @@ class Authenticator(ApplicationSession):
         # create database and attach tables to database slots
         #
         cbdir = self.config.extra['cbdir']
-        config = self.config.extra.get(u'database', {})
+        config = self.config.extra.get('database', {})
 
         dbpath = config.get('path', '.db-controller')
         assert type(dbpath) == str
@@ -206,13 +206,13 @@ class Authenticator(ApplicationSession):
         # Mailgun access key
         #
         access_key = None
-        if u'mailgun' in self.config.extra and \
-           u'access_key' in self.config.extra[u'mailgun'] and \
-           self.config.extra[u'mailgun'][u'access_key']:
-            access_key = self.config.extra[u'mailgun'][u'access_key']
+        if 'mailgun' in self.config.extra and \
+           'access_key' in self.config.extra['mailgun'] and \
+           self.config.extra['mailgun']['access_key']:
+            access_key = self.config.extra['mailgun']['access_key']
         else:
-            if u'MAILGUN_KEY' in os.environ:
-                access_key = os.environ[u'MAILGUN_KEY']
+            if 'MAILGUN_KEY' in os.environ:
+                access_key = os.environ['MAILGUN_KEY']
             else:
                 self.log.warn('Mailgun access key unconfigured (not in config, and no env var MAILGUN_KEY set)')
 
@@ -220,13 +220,13 @@ class Authenticator(ApplicationSession):
         # eg https://api.mailgun.net/v3/mailing.crossbar.io/messages
         #
         submit_url = None
-        if u'mailgun' in self.config.extra and \
-           u'submit_url' in self.config.extra[u'mailgun'] and \
-           self.config.extra[u'mailgun'][u'submit_url']:
-            submit_url = self.config.extra[u'mailgun'][u'submit_url']
+        if 'mailgun' in self.config.extra and \
+           'submit_url' in self.config.extra['mailgun'] and \
+           self.config.extra['mailgun']['submit_url']:
+            submit_url = self.config.extra['mailgun']['submit_url']
         else:
-            if u'MAILGUN_URL' in os.environ:
-                submit_url = os.environ[u'MAILGUN_URL']
+            if 'MAILGUN_URL' in os.environ:
+                submit_url = os.environ['MAILGUN_URL']
             else:
                 self.log.warn('Mailgun submit URL unconfigured (not in config, and no env var MAILGUN_URL set)')
 
@@ -241,7 +241,7 @@ class Authenticator(ApplicationSession):
 
         # register our dynamic authenticator
         try:
-            await self.register(self._authenticate, u'com.crossbario.fabric.authenticate')
+            await self.register(self._authenticate, 'com.crossbario.fabric.authenticate')
         except Exception:
             self.log.failure('failed to register dynamic authenticator: {log_failure}')
             raise Exception('fatal: failed to register dynamic authenticator')
@@ -268,36 +268,36 @@ class Authenticator(ApplicationSession):
 
         # proceed according to authmethod
         #
-        if u'authmethod' not in details:
-            msg = u'missing "authmethod" in authentication details (WAMP HELLO message details)'
+        if 'authmethod' not in details:
+            msg = 'missing "authmethod" in authentication details (WAMP HELLO message details)'
             raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                    Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
 
-        authmethod = details[u'authmethod']
+        authmethod = details['authmethod']
 
-        if authmethod not in [u'cryptosign']:
+        if authmethod not in ['cryptosign']:
             msg = 'authmethod "{}" not permissible'.format(authmethod)
             raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                    Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
 
-        if authmethod == u'cryptosign':
+        if authmethod == 'cryptosign':
 
             # extract mandatory public key
             #
-            if u'authextra' not in details or u'pubkey' not in details[u'authextra']:
-                msg = u'missing public key in authextra for authmethod cryptosign'
+            if 'authextra' not in details or 'pubkey' not in details['authextra']:
+                msg = 'missing public key in authextra for authmethod cryptosign'
                 raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                        Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
-            pubkey = details[u'authextra'][u'pubkey']
+            pubkey = details['authextra']['pubkey']
 
             # check requested authrole
             #
-            authrole = details.get(u'authrole', None)
+            authrole = details.get('authrole', None)
             if realm is None:
                 # this is either a node joining a management realm or a user joining
                 # the global user realm. a node must request authrole "node"
                 if authrole not in [None, Authenticator.MREALM_NODE_ROLE]:
-                    msg = u'invalid requested authrole "{}" for realm "{}"'.format(authrole, realm)
+                    msg = 'invalid requested authrole "{}" for realm "{}"'.format(authrole, realm)
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                            Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
 
@@ -305,13 +305,13 @@ class Authenticator(ApplicationSession):
                 # this is a user joining the global user realm - authrole is assigned
                 # automatically and cannot be requested explicitly
                 if authrole not in [None, Authenticator.GLOBAL_USER_REALM_USER_ROLE]:
-                    msg = u'invalid requested authrole "{}" for realm "{}"'.format(authrole, realm)
+                    msg = 'invalid requested authrole "{}" for realm "{}"'.format(authrole, realm)
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                            Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
             else:
                 # this is a user joining a specific management realm.
                 if authrole is not None and authrole not in Authenticator.MREALM_USER_ROLES:
-                    msg = u'invalid requested authrole "{}" for realm "{}"'.format(authrole, realm)
+                    msg = 'invalid requested authrole "{}" for realm "{}"'.format(authrole, realm)
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                            Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
 
@@ -319,7 +319,7 @@ class Authenticator(ApplicationSession):
             #
             if authrole == Authenticator.MREALM_NODE_ROLE:
                 if realm is not None:
-                    msg = u'invalid requested realm "{}" for node - nodes MUST NOT request a realm (the realm is auto-assigned based on Crossbar.io configuration)'.format(
+                    msg = 'invalid requested realm "{}" for node - nodes MUST NOT request a realm (the realm is auto-assigned based on Crossbar.io configuration)'.format(
                         realm)
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_PARAMETERS,
                                            Authenticator.ERROR_AUTH_INVALID_PARAMETERS_MSG.format(msg))
@@ -332,8 +332,8 @@ class Authenticator(ApplicationSession):
             else:
                 # a user may request to join different realms using the same pubkey, hence
                 # we forward all info the client supplied
-                activation_code = details[u'authextra'].get(u'activation_code', None)
-                request_new_activation_code = details[u'authextra'].get(u'request_new_activation_code', False)
+                activation_code = details['authextra'].get('activation_code', None)
+                request_new_activation_code = details['authextra'].get('request_new_activation_code', False)
                 auth = await self._auth_user(realm, authid, authrole, pubkey, activation_code,
                                              request_new_activation_code)
                 return auth
@@ -382,12 +382,12 @@ class Authenticator(ApplicationSession):
                 # assigned_auth_id = str(node_oid)
 
                 auth = {
-                    u'pubkey': pubkey,
-                    u'realm': mrealm.name,
-                    u'authid': assigned_auth_id,
-                    u'role': Authenticator.MREALM_NODE_ROLE,
-                    u'extra': node.authextra,
-                    u'cache': False
+                    'pubkey': pubkey,
+                    'realm': mrealm.name,
+                    'authid': assigned_auth_id,
+                    'role': Authenticator.MREALM_NODE_ROLE,
+                    'extra': node.authextra,
+                    'cache': False
                 }
 
                 self._connected_nodes[node.mrealm_oid][node_oid] = auth
@@ -397,7 +397,7 @@ class Authenticator(ApplicationSession):
                     pubkey=hlid('0x' + pubkey[:16] + '..'),
                     authid=hlid(auth['authid']),
                     authrole=hlid(auth['role']),
-                    realm=hlid(auth[u'realm']),
+                    realm=hlid(auth['realm']),
                     func=hltype(self._auth_node))
                 return auth
 
@@ -481,12 +481,12 @@ class Authenticator(ApplicationSession):
         # superusers are treated special ..
         if pubkey in self._superusers:
             auth = {
-                u'pubkey': pubkey,
-                u'realm': realm,
-                u'authid': 'superuser',
-                u'role': authrole,
-                u'extra': None,
-                u'cache': False
+                'pubkey': pubkey,
+                'realm': realm,
+                'authid': 'superuser',
+                'role': authrole,
+                'extra': None,
+                'cache': False
             }
             self.log.info(
                 hl('SUPERUSER authenticated (realm={}, authid={}, authrole={})'.format(
@@ -556,12 +556,12 @@ class Authenticator(ApplicationSession):
                 # but ignore any authrole that might have been requested
                 if realm is None or realm == Authenticator.GLOBAL_USER_REALM:
                     auth = {
-                        u'pubkey': pubkey,
-                        u'realm': Authenticator.GLOBAL_USER_REALM,
-                        u'authid': authid,
-                        u'role': Authenticator.GLOBAL_USER_REALM_USER_ROLE,
-                        u'extra': None,
-                        u'cache': False
+                        'pubkey': pubkey,
+                        'realm': Authenticator.GLOBAL_USER_REALM,
+                        'authid': authid,
+                        'role': Authenticator.GLOBAL_USER_REALM_USER_ROLE,
+                        'extra': None,
+                        'cache': False
                     }
                     self.log.info('Found user {authid} with active pubkey, authenticating for global user realm',
                                   authid=authid)
@@ -616,12 +616,12 @@ class Authenticator(ApplicationSession):
                     authrole = MAP.get(authrole, None)
 
                     auth = {
-                        u'pubkey': pubkey,
-                        u'realm': realm,
-                        u'authid': authid,
-                        u'role': authrole,
-                        u'extra': None,
-                        u'cache': False
+                        'pubkey': pubkey,
+                        'realm': realm,
+                        'authid': authid,
+                        'role': authrole,
+                        'extra': None,
+                        'cache': False
                     }
                     self.log.info('auth=\n{auth}', auth=auth)
 
@@ -645,7 +645,7 @@ class Authenticator(ApplicationSession):
                                            Authenticator.ERROR_AUTH_PENDING_ACT_MSG.format(passed_secs_str))
 
                 if activation_code != activation.code:
-                    msg = u'code does not match pending one'
+                    msg = 'code does not match pending one'
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_ACT_CODE,
                                            Authenticator.ERROR_AUTH_INVALID_ACT_CODE_MSG.format(msg))
 
@@ -659,23 +659,23 @@ class Authenticator(ApplicationSession):
                         else:
                             raise Exception('no such activation')
 
-                    msg = u'code created {} ago has expired'.format(passed_secs_str)
+                    msg = 'code created {} ago has expired'.format(passed_secs_str)
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_ACT_CODE,
                                            Authenticator.ERROR_AUTH_INVALID_ACT_CODE_MSG.format(msg))
 
                 # sanitize the stored activation info against what the client provided
                 if activation.atype not in [ActivationType.LOGIN, ActivationType.REGISTRATION]:
-                    msg = u'activation type "{}" is not for user login/registration.'.format(activation.atype)
+                    msg = 'activation type "{}" is not for user login/registration.'.format(activation.atype)
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_ACT_CODE,
                                            Authenticator.ERROR_AUTH_INVALID_ACT_CODE_MSG.format(msg))
 
                 if activation.email != authid:
-                    msg = u'email associated with activation code does not match authid provided by client.'
+                    msg = 'email associated with activation code does not match authid provided by client.'
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_ACT_CODE,
                                            Authenticator.ERROR_AUTH_INVALID_ACT_CODE_MSG.format(msg))
 
                 if activation.pubkey != pubkey:
-                    msg = u'pubkey associated with activation code does not match pubkey provided by client.'
+                    msg = 'pubkey associated with activation code does not match pubkey provided by client.'
                     raise ApplicationError(Authenticator.ERROR_AUTH_INVALID_ACT_CODE,
                                            Authenticator.ERROR_AUTH_INVALID_ACT_CODE_MSG.format(msg))
 
@@ -706,17 +706,17 @@ class Authenticator(ApplicationSession):
 
                 # immediately auth user on global users realm
                 auth = {
-                    u'pubkey': pubkey,
-                    u'realm': Authenticator.GLOBAL_USER_REALM,
-                    u'authid': authid,
-                    u'role': Authenticator.GLOBAL_USER_REALM_USER_ROLE,
-                    u'extra': None,
-                    u'cache': False
+                    'pubkey': pubkey,
+                    'realm': Authenticator.GLOBAL_USER_REALM,
+                    'authid': authid,
+                    'role': Authenticator.GLOBAL_USER_REALM_USER_ROLE,
+                    'extra': None,
+                    'cache': False
                 }
 
                 self.log.info('found principal for public key {pubkey} of {authid}',
                               pubkey=pubkey,
-                              authid=auth[u'authid'])
+                              authid=auth['authid'])
 
                 return auth
 
@@ -724,13 +724,13 @@ class Authenticator(ApplicationSession):
                 raise Exception('internal error: unprocessed activation status {}'.format(activation.status))
 
     def _send_user_login_mail(self, receiver, activation_code):
-        subject = u'Crossbar.io: your LOGIN code'
-        text = u'We have received a login request for your account. Please use this activation code: {}'.format(
+        subject = 'Crossbar.io: your LOGIN code'
+        text = 'We have received a login request for your account. Please use this activation code: {}'.format(
             activation_code)
         return self._messenger.send_message(receiver, subject, text)
 
     def _send_user_registration_mail(self, receiver, activation_code):
-        subject = u'Crossbar.io: your REGISTRATION code'
-        text = u'We have received a registration request for your account. Please use this activation code: {}'.format(
+        subject = 'Crossbar.io: your REGISTRATION code'
+        text = 'We have received a registration request for your account. Please use this activation code: {}'.format(
             activation_code)
         return self._messenger.send_message(receiver, subject, text)
