@@ -38,7 +38,7 @@ from txaio import time_ns
 from crossbar._util import hlid, hl, hlval
 from crossbar.node.main import _get_versions
 from crossbar.common import checkconfig
-from crossbar.common.key import _read_node_key, _read_release_key, _write_node_key, _parse_key_file
+from crossbar.common.key import _read_node_key, _read_release_key, _write_node_key, _parse_node_key
 
 from cfxdb.globalschema import GlobalSchema
 from cfxdb.mrealmschema import MrealmSchema
@@ -82,7 +82,7 @@ class DomainManager(object):
         """
         assert isinstance(details, CallDetails)
 
-        self.log.info('{klass}.get_status(details={details})', klass=self.__class__.__name__, details=details)
+        self.log.debug('{klass}.get_status(details={details})', klass=self.__class__.__name__, details=details)
 
         now = utcnow()
         uptime_secs = (iso8601.parse_date(now) - iso8601.parse_date(self._session._started)).total_seconds()
@@ -112,7 +112,7 @@ class DomainManager(object):
         """
         assert isinstance(details, CallDetails)
 
-        self.log.info('{klass}.get_version(details={details})', klass=self.__class__.__name__, details=details)
+        self.log.debug('{klass}.get_version(details={details})', klass=self.__class__.__name__, details=details)
 
         # FIXME
         from twisted.internet import reactor
@@ -135,7 +135,7 @@ class DomainManager(object):
         """
         assert isinstance(details, CallDetails)
 
-        self.log.info('{klass}.get_license(details={details})', klass=self.__class__.__name__, details=details)
+        self.log.debug('{klass}.get_license(details={details})', klass=self.__class__.__name__, details=details)
 
         # FIXME: check blockchain for license information
         license = {
@@ -180,7 +180,7 @@ class DomainController(ApplicationSession):
 
         self._meterurl = checkconfig.maybe_from_env('metering.submit.url', self._meterurl)
 
-        self.log.info(
+        self.log.debug(
             'Initialized {klass} from ComponentConfig.extra [node_key="{node_key}", meter_url="{meter_url}"]',
             klass=self.__class__.__name__,
             meter_url=self._meterurl,
@@ -216,10 +216,10 @@ class DomainController(ApplicationSession):
         self.db.__enter__()
         self.schema = GlobalSchema.attach(self.db)
 
-        self.log.info('{klass} global database opened [dbpath={dbpath}, maxsize={maxsize}]',
-                      klass=self.__class__.__name__,
-                      dbpath=hlid(dbpath),
-                      maxsize=hlid(maxsize))
+        self.log.debug('{klass} global database opened [dbpath={dbpath}, maxsize={maxsize}]',
+                       klass=self.__class__.__name__,
+                       dbpath=hlid(dbpath),
+                       maxsize=hlid(maxsize))
 
     async def onJoin(self, details):
 
@@ -265,8 +265,8 @@ class DomainController(ApplicationSession):
             with self.db.begin(write=True) as txn:
                 self._default_mrealm_oid = self.schema.idx_mrealms_by_name[txn, mrealm_name]
                 if self._default_mrealm_oid:
-                    self.log.info('ok, default management realm already exists ({oid})',
-                                  oid=hlid(self._default_mrealm_oid))
+                    self.log.debug('ok, default management realm already exists ({oid})',
+                                   oid=hlid(self._default_mrealm_oid))
 
                     # This should not happen - but better verify that the owner of the default management realm
                     # is the current superuser.
@@ -351,10 +351,11 @@ class DomainController(ApplicationSession):
                                         if file == 'key.pub':
                                             node_key_file = os.path.join(r, file)
                                             if os.path.isfile(node_key_file):
-                                                node_key_tags = _parse_key_file(node_key_file)
+                                                node_key_tags = _parse_node_key(node_key_file)
                                                 node_key_hex = node_key_tags['public-key-ed25519']
                                                 node_id = node_key_tags.get('node-authid', None)
-                                                pubkeys.append((r, node_key_hex, node_id))
+                                                cluster_ip = node_key_tags.get('node-cluster-ip', None)
+                                                pubkeys.append((r, node_key_hex, node_id, cluster_ip))
 
                             self.log.debug(
                                 '{klass}::watch_and_pair: found {cnt} directories (matching), with {cntk} node keys scanned ..',
@@ -366,10 +367,10 @@ class DomainController(ApplicationSession):
                                 # determine actually new pubkeys
                                 new_pubkeys = []
                                 with self.db.begin() as txn:
-                                    for cbdir, pubkey, node_id in pubkeys:
+                                    for cbdir, pubkey, node_id, cluster_ip in pubkeys:
                                         node_oid = self.schema.idx_nodes_by_pubkey[txn, pubkey]
                                         if not node_oid:
-                                            new_pubkeys.append((cbdir, pubkey, node_id))
+                                            new_pubkeys.append((cbdir, pubkey, node_id, cluster_ip))
                                         else:
                                             node = self.schema.nodes[txn, node_oid]
                                             self.log.debug(
@@ -388,10 +389,11 @@ class DomainController(ApplicationSession):
                                     klass=self.__class__.__name__)
 
                                 # store all new pubkeys
-                                for cbdir, pubkey, node_id in new_pubkeys:
+                                for cbdir, pubkey, node_id, cluster_ip in new_pubkeys:
                                     node = Node()
                                     node.oid = uuid.uuid4()
                                     node.pubkey = pubkey
+                                    node.cluster_ip = cluster_ip
 
                                     # auto-pair newly discovered node to the default management realm, and owned by superuser
                                     node.owner_oid = self._superuser_oid
@@ -402,6 +404,7 @@ class DomainController(ApplicationSession):
                                         node.authid = 'node-{}'.format(str(node.oid)[:8])
                                     node.authextra = {
                                         'node_oid': str(node.oid),
+                                        'cluster_ip': cluster_ip,
                                         'mrealm_oid': str(node.mrealm_oid),
                                     }
 
@@ -428,6 +431,7 @@ class DomainController(ApplicationSession):
                                                 ('management-realm', 'default'),
                                                 ('management-realm-oid', str(node.mrealm_oid)),
                                                 ('node-oid', str(node.oid)),
+                                                ('node-cluster-ip', node.cluster_ip),
                                                 ('node-authid', str(node.authid)),
                                                 ('activation-code', activation_code),
                                                 ('public-key-ed25519', pubkey),
@@ -561,7 +565,7 @@ class DomainController(ApplicationSession):
                 self.log.warn('Master heartbeat loop iteration {tick} finished: excessive run-time of {duration} ms!',
                               tick=self._tick,
                               duration=duration)
-            self.log.info(
+            self.log.debug(
                 'Master heartbeat loop iteration {tick} finished in {duration} ms (database {used} used, {free}% free)',
                 tick=self._tick,
                 free=round(free * 100., 2),
@@ -575,12 +579,12 @@ class DomainController(ApplicationSession):
         master_tick_period = 300
         c = LoopingCall(tick)
         c.start(master_tick_period)
-        self.log.info('Master heartbeat loop started .. [period={period} secs]', period=master_tick_period)
+        self.log.debug('Master heartbeat loop started .. [period={period} secs]', period=master_tick_period)
 
         # note status started
         #
         self._started = utcnow()
-        self.log.info('Domain controller ready (realm="{realm}")!', realm=hlid(self._realm))
+        self.log.debug('Domain controller ready (realm="{realm}")!', realm=hlid(self._realm))
 
     def _first_metering(self, mrealm_id):
         """
@@ -1018,7 +1022,7 @@ class DomainController(ApplicationSession):
                                            usage.marshal(),
                                            options=PublishOptions(acknowledge=True))
 
-                        self.log.info(
+                        self.log.debug(
                             'Usage metering: aggregated and stored period from {from_ts} to {until_ts} ({duration}) usage metering data for mrealm "{mrealm_id}"',
                             mrealm_id=mrealm_id,
                             duration=str(np.timedelta64(until_ts - from_ts, 's')),
@@ -1030,14 +1034,14 @@ class DomainController(ApplicationSession):
         else:
             self.log.debug('Usage metering: no new intervals to aggregate.')
 
-        self.log.info('Usage metering: finished aggregating [{cnt_new} intervals stored]', cnt_new=cnt_new)
+        self.log.debug('Usage metering: finished aggregating [{cnt_new} intervals stored]', cnt_new=cnt_new)
 
         return cnt_new
 
     @inlineCallbacks
     def _submit_metering(self, started, filter_status=[1], limit=None):
-        self.log.info('Usage metering: submitting metering records .. [started="{started}"]',
-                      started=np.datetime64(started, 'ns'))
+        self.log.debug('Usage metering: submitting metering records .. [started="{started}"]',
+                       started=np.datetime64(started, 'ns'))
 
         # collects keys in table "schema.usage" for metering records to be submitted
         keys = []
@@ -1061,9 +1065,9 @@ class DomainController(ApplicationSession):
             with self.db.begin() as txn:
                 rec = self.schema.usage[txn, key]
 
-            self.log.info('Usage metering: submitting metering record to "{url}"\n{rec}',
-                          rec=pprint.pformat(rec.marshal()),
-                          url=self._meterurl)
+            self.log.debug('Usage metering: submitting metering record to "{url}"\n{rec}',
+                           rec=pprint.pformat(rec.marshal()),
+                           url=self._meterurl)
 
             # the master node public key
             verify_key = self._node_key.verify_key.encode(encoder=nacl.encoding.RawEncoder)
@@ -1131,13 +1135,13 @@ class DomainController(ApplicationSession):
             with self.db.begin(write=True) as txn:
                 self.schema.usage[txn, key] = rec
 
-            self.log.info(
+            self.log.debug(
                 'Usage metering: metering record for "{timestamp}" processed with new status {status} [metering_id="{metering_id}"].',
                 timestamp=rec.timestamp,
                 metering_id=metering_id,
                 status=rec.status)
 
-        self.log.info(
+        self.log.debug(
             'Usage metering: finished submitting metering records [tried={tried}, success={success}, failed={failed}]',
             tried=tried,
             success=success,
