@@ -13,7 +13,7 @@ from twisted.internet.defer import succeed
 from txaio import make_logger
 
 from autobahn.wamp.exception import ApplicationError
-from autobahn.wamp.cryptosign import SigningKey
+from autobahn.wamp.cryptosign import CryptosignKey
 from autobahn.wamp.types import CallOptions, ComponentConfig
 
 from crossbar._util import hltype, hlid, hluserid, hl
@@ -23,7 +23,7 @@ from crossbar.router.session import RouterSessionFactory
 from crossbar.router.service import RouterServiceAgent
 from crossbar.worker.types import RouterRealm
 from crossbar.common.checkconfig import NODE_SHUTDOWN_ON_WORKER_EXIT
-from crossbar.common.key import _maybe_generate_key
+from crossbar.common.key import _maybe_generate_node_key
 from crossbar.node.controller import NodeController
 
 
@@ -43,14 +43,14 @@ class Node(object):
 
     ROUTER_SERVICE = RouterServiceAgent
 
+    CONFIG_SOURCE_EMPTY = 0
     CONFIG_SOURCE_DEFAULT = 1
-    CONFIG_SOURCE_EMPTY = 2
-    CONFIG_SOURCE_LOCALFILE = 3
+    CONFIG_SOURCE_LOCALFILE = 2
     CONFIG_SOURCE_XBRNETWORK = 4
     CONFIG_SOURCE_TO_STR = {
+        0: 'empty',
         1: 'default',
-        2: 'empty',
-        3: 'localfile',
+        2: 'localfile',
         4: 'xbrnetwork',
     }
 
@@ -95,10 +95,13 @@ class Node(object):
         # the node controller realm
         self._realm = 'crossbar'
 
-        # config of this node.
+        # config of this node
         self._config = None
 
-        # node private key :class:`autobahn.wamp.cryptosign.SigningKey`
+        # source(s) of the config of this node
+        self._config_source = 0
+
+        # node private key :class:`autobahn.wamp.cryptosign.CryptosignKey`
         self._node_key = None
 
         # when running in managed mode, this will hold the session to CFC
@@ -150,7 +153,7 @@ class Node(object):
         Returns the node (private signing) key pair.
 
         :return: The node key.
-        :rtype: :class:`autobahn.wamp.cryptosign.SigningKey`
+        :rtype: :class:`autobahn.wamp.cryptosign.CryptosignKey`
         """
         return self._node_key
 
@@ -163,7 +166,7 @@ class Node(object):
 
         IMPORTANT: this function is run _before_ start of Twisted reactor!
         """
-        was_new, self._node_key = _maybe_generate_key(cbdir)
+        was_new, self._node_key = _maybe_generate_node_key(cbdir)
         return was_new
 
     def load_config(self, configfile=None, default=None):
@@ -204,7 +207,7 @@ class Node(object):
             keyring_type = self._config['controller']['keyring']['type']
             if keyring_type == 'file':
                 key_path = self._config['controller']['keyring']['path']
-                self._node_key = SigningKey.from_raw_key(key_path)
+                self._node_key = CryptosignKey.from_file(key_path)
             else:
                 raise RuntimeError("NotImplemented: hsm authentication is currently not implemented.")
 
@@ -450,9 +453,13 @@ class Node(object):
         self._boot_complete = Deferred()
 
         # startup the node personality ..
-        self.log.info('{func}::NODE_BOOT_BEGIN', func=hltype(self.personality.Node.boot))
+        self.log.info('{func}::NODE_BOOT_BEGIN[node_id="{node_id}"]',
+                      node_id=hlid(self._node_id),
+                      func=hltype(self.personality.Node.boot))
         res = yield self.personality.Node.boot(self)
-        self.log.info('{func}::NODE_BOOT_COMPLETE', func=hltype(self.personality.Node.boot))
+        self.log.info('{func}::NODE_BOOT_COMPLETE[node_id="{node_id}"]',
+                      func=hltype(self.personality.Node.boot),
+                      node_id=hlid(self._node_id))
 
         # notify observers of boot completition
         self._boot_complete.callback(res)
@@ -875,7 +882,42 @@ class Node(object):
     def _configure_native_worker_proxy(self, worker_logname, worker_id, worker):
         yield self._configure_native_worker_common(worker_logname, worker_id, worker)
 
+        # set up backend connections on the proxy
+
+        for i, connection_name in enumerate(worker.get('connections', {})):
+            self.log.debug(
+                "Starting connection {index}: {name}",
+                index=i,
+                name=connection_name,
+            )
+            yield self._controller.call(
+                'crossbar.worker.{}.start_proxy_connection'.format(worker_id),
+                connection_name,
+                worker['connections'].get(connection_name, {}),
+            )
+
+        # set up realms and roles on the proxy
+
+        for i, realm_name in enumerate(worker.get('routes', {})):
+            roles = worker['routes'][realm_name]
+            for role_id, connections in roles.items():
+                if not isinstance(connections, list):
+                    connections = [connections]  # used to be a single string, now a list of strings
+                for connection_id in connections:
+                    self.log.debug(
+                        "Starting proxy realm route {realm}, {role} to {connection}",
+                        realm=realm_name,
+                        role=role_id,
+                        connection=connection_id,
+                    )
+                    yield self._controller.call(
+                        'crossbar.worker.{}.start_proxy_realm_route'.format(worker_id),
+                        realm_name,
+                        {role_id: connection_id},
+                    )
+
         # start transports on proxy
+
         for i, transport in enumerate(worker.get('transports', [])):
 
             if 'id' in transport:
@@ -942,37 +984,3 @@ class Node(object):
                 worker_logname=worker_logname,
                 transport_id=hlid(transport_id),
             )
-
-        # set up backend connections on the proxy
-
-        for i, connection_name in enumerate(worker.get('connections', {})):
-            self.log.debug(
-                "Starting connection {index}: {name}",
-                index=i,
-                name=connection_name,
-            )
-            yield self._controller.call(
-                'crossbar.worker.{}.start_proxy_connection'.format(worker_id),
-                connection_name,
-                worker['connections'].get(connection_name, {}),
-            )
-
-        # set up realms and roles on the proxy
-
-        for i, realm_name in enumerate(worker.get('routes', {})):
-            roles = worker['routes'][realm_name]
-            for role_id, connections in roles.items():
-                if not isinstance(connections, list):
-                    connections = [connections]  # used to be a single string, now a list of strings
-                for connection_id in connections:
-                    self.log.debug(
-                        "Starting proxy realm route {realm}, {role} to {connection}",
-                        realm=realm_name,
-                        role=role_id,
-                        connection=connection_id,
-                    )
-                    yield self._controller.call(
-                        'crossbar.worker.{}.start_proxy_realm_route'.format(worker_id),
-                        realm_name,
-                        {role_id: connection_id},
-                    )
