@@ -7,21 +7,24 @@
 
 import os
 import binascii
+from pprint import pformat
+from typing import Optional, Union, Dict, Any
 
 import nacl
 from nacl.signing import VerifyKey
 from nacl.exceptions import BadSignatureError
 
-from autobahn import util
-from autobahn.wamp import types
-from autobahn.wamp.exception import ApplicationError
-
 from twisted.internet.defer import Deferred
 
 import txaio
 
-from crossbar._util import hltype, hlid
+from autobahn import util
+from autobahn.util import hltype, hlid, hlval
+from autobahn.wamp.types import Accept, Deny, HelloDetails, Challenge, TransportDetails
+from autobahn.wamp.exception import ApplicationError
+
 from crossbar.router.auth.pending import PendingAuth
+from crossbar.interfaces import IRealmContainer, IPendingAuth
 
 __all__ = (
     'PendingAuthCryptosign',
@@ -38,26 +41,23 @@ class PendingAuthCryptosign(PendingAuth):
 
     AUTHMETHOD = 'cryptosign'
 
-    def __init__(self, pending_session_id, transport_info, realm_container, config):
+    def __init__(self, pending_session_id: int, transport_details: TransportDetails, realm_container: IRealmContainer,
+                 config: Dict[str, Any]):
         super(PendingAuthCryptosign, self).__init__(
             pending_session_id,
-            transport_info,
+            transport_details,
             realm_container,
             config,
         )
-        self._verify_key = None
-
         # https://tools.ietf.org/html/rfc5056
         # https://tools.ietf.org/html/rfc5929
         # https://www.ietf.org/proceedings/90/slides/slides-90-uta-0.pdf
-        channel_id_hex = transport_info.get('channel_id', None)
-        if channel_id_hex:
-            self._channel_id = binascii.a2b_hex(channel_id_hex)
-        else:
-            self._channel_id = None
+        self._channel_id = transport_details.channel_id.get('tls-unique',
+                                                            None) if transport_details.channel_id else None
 
-        self._challenge = None
-        self._expected_signed_message = None
+        self._verify_key: Optional[VerifyKey] = None
+        self._challenge: Optional[bytes] = None
+        self._expected_signed_message: Optional[bytes] = None
 
         # create a map: pubkey -> authid
         # this is to allow clients to authenticate without specifying an authid
@@ -67,35 +67,41 @@ class PendingAuthCryptosign(PendingAuth):
                 for pubkey in principal['authorized_keys']:
                     self._pubkey_to_authid[pubkey] = authid
 
-    def _compute_challenge(self, channel_binding):
+    def _compute_challenge(self, requested_channel_binding: Optional[str]) -> Dict[str, Any]:
         self._challenge = os.urandom(32)
-
-        if self._channel_id and channel_binding:
+        if self._channel_id and requested_channel_binding == 'tls-unique':
             self._expected_signed_message = util.xor(self._challenge, self._channel_id)
         else:
             self._expected_signed_message = self._challenge
 
         extra = {
-            'challenge': binascii.b2a_hex(self._challenge).decode('ascii'),
-            'channel_binding': channel_binding,
+            'challenge': binascii.b2a_hex(self._challenge).decode(),
+            'channel_binding': requested_channel_binding,
         }
+        self.log.info(
+            '{func}::_compute_challenge(channel_binding={channel_binding})[channel_id={channel_id}] -> extra=\n{extra}',
+            func=hltype(self.hello),
+            channel_id=hlid('0x' + binascii.b2a_hex(self._channel_id).decode()) if self._channel_id else None,
+            channel_binding=hlval('"' + requested_channel_binding +
+                                  '"') if requested_channel_binding is not None else None,
+            extra=pformat(extra))
         return extra
 
-    def hello(self, realm: str, details: types.HelloDetails):
-        self.log.debug('{func}::hello(realm="{realm}", details.authid="{authid}", details.authrole="{authrole}")',
-                       func=hltype(self.hello),
-                       realm=hlid(realm),
-                       authid=hlid(details.authid),
-                       authrole=hlid(details.authrole))
+    def hello(self, realm: str, details: HelloDetails) -> Union[Accept, Deny, Challenge]:
+        self.log.info('{func}::hello(realm="{realm}", details.authid="{authid}", details.authrole="{authrole}")',
+                      func=hltype(self.hello),
+                      realm=hlid(realm),
+                      authid=hlid(details.authid),
+                      authrole=hlid(details.authrole))
 
         # the channel binding requested by the client authenticating
-        channel_binding = details.authextra.get('channel_binding', None) if details.authextra else None
-        if channel_binding is not None and channel_binding not in ['tls-unique']:
-            return types.Deny(message='invalid channel binding type "{}" requested'.format(channel_binding))
+        requested_channel_binding = details.authextra.get('channel_binding', None) if details.authextra else None
+        if requested_channel_binding is not None and requested_channel_binding not in ['tls-unique']:
+            return Deny(message='invalid channel binding type "{}" requested'.format(requested_channel_binding))
         else:
-            self.log.debug(
+            self.log.info(
                 "WAMP-cryptosign CHANNEL BINDING requested: channel_binding={channel_binding}, channel_id={channel_id}",
-                channel_binding=channel_binding,
+                channel_binding=requested_channel_binding,
                 channel_id=self._channel_id)
 
         # remember the realm the client requested to join (if any)
@@ -128,21 +134,28 @@ class PendingAuthCryptosign(PendingAuth):
                             if self._authid is None:
                                 self._authid = _authid
                             else:
-                                return types.Deny(message='cannot infer client identity from pubkey: multiple authids '
-                                                  'in principal database have this pubkey')
+                                return Deny(message='cannot infer client identity from pubkey: multiple authids '
+                                            'in principal database have this pubkey')
                     if self._authid is None:
-                        return types.Deny(message='cannot identify client: no authid requested and no principal found '
-                                          'for provided extra.pubkey')
+                        return Deny(message='cannot identify client: no authid requested and no principal found '
+                                    'for provided extra.pubkey')
                 else:
-                    return types.Deny(
-                        message='cannot identify client: no authid requested and no extra.pubkey provided')
+                    return Deny(message='cannot identify client: no authid requested and no extra.pubkey provided')
 
-            if self._authid in self._config.get('principals', {}):
+            principals = self._config.get('principals', {})
+            if self._authid in principals:
 
-                principal = self._config['principals'][self._authid]
+                principal = principals[self._authid]
 
                 if pubkey and (pubkey not in principal['authorized_keys']):
-                    return types.Deny(
+                    self.log.warn(
+                        'extra.pubkey {pubkey} provided does not match any one of authorized_keys for the principal [func="{func}"]:\n{principals}',
+                        func=hltype(self.hello),
+                        realm=hlid(realm),
+                        authid=hlid(details.authid),
+                        pubkey=hlval(pubkey),
+                        principals=pformat(principals))
+                    return Deny(
                         message='extra.pubkey provided does not match any one of authorized_keys for the principal')
 
                 error = self._assign_principal(principal)
@@ -151,11 +164,17 @@ class PendingAuthCryptosign(PendingAuth):
 
                 self._verify_key = VerifyKey(pubkey, encoder=nacl.encoding.HexEncoder)
 
-                extra = self._compute_challenge(channel_binding)
-                return types.Challenge(self._authmethod, extra)
+                extra = self._compute_challenge(requested_channel_binding)
+                return Challenge(self._authmethod, extra)
 
             else:
-                return types.Deny(message='no principal with authid "{}" exists'.format(details.authid))
+                self.log.warn(
+                    'no principal with authid "{authid}" exists in principals for realm "{realm}" [func="{func}"]:\n{principals}',
+                    func=hltype(self.hello),
+                    realm=hlid(realm),
+                    authid=hlid(self._authid),
+                    principals=pformat(principals))
+                return Deny(message='no principal with authid "{}" exists'.format(self._authid))
 
         elif self._config['type'] == 'dynamic':
 
@@ -193,28 +212,28 @@ class PendingAuthCryptosign(PendingAuth):
                         realm=realm,
                         details=details,
                         principal=principal)
-                    error = self._assign_principal(principal)
-                    if error:
-                        d.callback(error)
+                    _error = self._assign_principal(principal)
+                    if _error:
+                        d.callback(_error)
                         return
 
                     self._verify_key = VerifyKey(principal['pubkey'], encoder=nacl.encoding.HexEncoder)
 
-                    extra = self._compute_challenge(channel_binding)
-                    d.callback(types.Challenge(self._authmethod, extra))
+                    extra = self._compute_challenge(requested_channel_binding)
+                    d.callback(Challenge(self._authmethod, extra))
 
-                def on_authenticate_error(err):
+                def on_authenticate_error(_error):
                     self.log.debug(
-                        '{klass}.hello(realm="{realm}", details={details}) -> on_authenticate_error(err={err})',
+                        '{klass}.hello(realm="{realm}", details={details}) -> on_authenticate_error(error={error})',
                         klass=self.__class__.__name__,
                         realm=realm,
                         details=details,
-                        err=err)
+                        error=_error)
                     try:
-                        d.callback(self._marshal_dynamic_authenticator_error(err))
+                        d.callback(self._marshal_dynamic_authenticator_error(_error))
                     except:
                         self.log.failure()
-                        d.callback(error)
+                        d.callback(_error)
 
                 d2.addCallbacks(on_authenticate_ok, on_authenticate_error)
                 return d2
@@ -232,9 +251,9 @@ class PendingAuthCryptosign(PendingAuth):
 
             init_d = txaio.as_future(self._init_function_authenticator)
 
-            def init(error):
-                if error:
-                    return error
+            def init(_error):
+                if _error:
+                    return _error
 
                 self._session_details['authmethod'] = self._authmethod  # from AUTHMETHOD, via base
                 self._session_details['authid'] = details.authid
@@ -244,23 +263,23 @@ class PendingAuthCryptosign(PendingAuth):
                 auth_d = txaio.as_future(self._authenticator, realm, details.authid, self._session_details)
 
                 def on_authenticate_ok(principal):
-                    self.log.info(
+                    self.log.debug(
                         '{klass}.hello(realm="{realm}", details={details}) -> on_authenticate_ok(principal={principal})',
                         klass=self.__class__.__name__,
                         realm=realm,
                         details=details,
                         principal=principal)
-                    error = self._assign_principal(principal)
-                    if error:
-                        return error
+                    _error = self._assign_principal(principal)
+                    if _error:
+                        return _error
 
                     self._verify_key = VerifyKey(principal['pubkey'], encoder=nacl.encoding.HexEncoder)
 
-                    extra = self._compute_challenge(channel_binding)
-                    return types.Challenge(self._authmethod, extra)
+                    _extra = self._compute_challenge(requested_channel_binding)
+                    return Challenge(self._authmethod, _extra)
 
                 def on_authenticate_error(err):
-                    self.log.info(
+                    self.log.warn(
                         '{klass}.hello(realm="{realm}", details={details}) -> on_authenticate_error(err={err})',
                         klass=self.__class__.__name__,
                         realm=realm,
@@ -269,10 +288,10 @@ class PendingAuthCryptosign(PendingAuth):
                     try:
                         return self._marshal_dynamic_authenticator_error(err)
                     except Exception as e:
-                        error = ApplicationError.AUTHENTICATION_FAILED
+                        _error = ApplicationError.AUTHENTICATION_FAILED
                         message = 'marshalling of function-based authenticator error return failed: {}'.format(e)
                         self.log.warn('{klass}.hello.on_authenticate_error() - {msg}', msg=message)
-                        return types.Deny(error, message)
+                        return Deny(_error, message)
 
                 auth_d.addCallbacks(on_authenticate_ok, on_authenticate_error)
                 return auth_d
@@ -282,46 +301,50 @@ class PendingAuthCryptosign(PendingAuth):
 
         else:
             # should not arrive here, as config errors should be caught earlier
-            return types.Deny(message='invalid authentication configuration (authentication type "{}" is unknown)'.
-                              format(self._config['type']))
+            return Deny(message='invalid authentication configuration (authentication type "{}" is unknown)'.format(
+                self._config['type']))
 
-    def authenticate(self, signed_message):
+    def authenticate(self, signature: str) -> Union[Accept, Deny]:
         """
         Verify the signed message sent by the client. With WAMP-cryptosign, this must be 96 bytes (as a string
         in HEX encoding): the concatenation of the Ed25519 signature (64 bytes) and the 32 bytes we sent
         as a challenge previously, XORed with the 32 bytes transport channel ID (if available).
         """
         try:
-            if not isinstance(signed_message, str):
-                return types.Deny(message='invalid type {} for signed message'.format(type(signed_message)))
+            if not isinstance(signature, str):
+                return Deny(message='invalid type {} for signed message'.format(type(signature)))
 
             try:
-                signed_message = binascii.a2b_hex(signed_message)
+                signed_message = binascii.a2b_hex(signature)
             except TypeError:
-                return types.Deny(message='signed message is invalid (not a HEX encoded string)')
+                return Deny(message='signed message is invalid (not a HEX encoded string)')
 
             if len(signed_message) != 96:
-                return types.Deny(message='signed message has invalid length (was {}, but should have been 96)'.format(
+                return Deny(message='signed message has invalid length (was {}, but should have been 96)'.format(
                     len(signed_message)))
 
-            # now verify the signed message versus the client public key ..
+            # now verify the signed message versus the client public key
+            assert self._verify_key
             try:
                 message = self._verify_key.verify(signed_message)
             except BadSignatureError:
-                return types.Deny(message='signed message has invalid signature')
+                return Deny(message='signed message has invalid signature')
 
-            # .. and check that the message signed by the client is really what we expect
+            # and check that the message signed by the client is really what we expect
+            assert self._expected_signed_message
             if message != self._expected_signed_message:
-                return types.Deny(message='message signed is bogus')
+                return Deny(message='message signed is bogus [got 0x{}, expected 0x{}]'.format(
+                    binascii.b2a_hex(message).decode(),
+                    binascii.b2a_hex(self._expected_signed_message).decode()))
 
             # signature was valid _and_ the message that was signed is equal to
             # what we expected => accept the client
             return self._accept()
 
-        # should not arrive here .. but who knows
+        # should not arrive here, but who knows
         except Exception as e:
             self.log.failure()
-            return types.Deny(message='INTERNAL ERROR ({})'.format(e))
+            return Deny(message='INTERNAL ERROR ({})'.format(e))
 
 
 class PendingAuthCryptosignProxy(PendingAuthCryptosign):
@@ -338,34 +361,35 @@ class PendingAuthCryptosignProxy(PendingAuthCryptosign):
                        realm=realm,
                        details=details)
         if not details.authextra:
-            return types.Deny(message='missing required details.authextra')
+            return Deny(message='missing required details.authextra')
         for attr in ['proxy_authid', 'proxy_authrole', 'proxy_realm']:
             if attr not in details.authextra:
-                return types.Deny(message='missing required attribute {} in details.authextra'.format(attr))
+                return Deny(message='missing required attribute {} in details.authextra'.format(attr))
 
         if details.authrole is None:
             details.authrole = details.authextra.get('proxy_authrole', None)
         if details.authid is None:
             details.authid = details.authextra.get('proxy_authid', None)
 
-        # with authentictors of type "*-proxy", the principal returned in authenticating the
+        # with authenticators of type "*-proxy", the principal returned in authenticating the
         # incoming backend connection is ignored ..
-        f = super(PendingAuthCryptosignProxy, self).hello(realm, details)
+        f = txaio.as_future(super(PendingAuthCryptosignProxy, self).hello, realm, details)
 
         def assign(res):
-            """
-            .. and the incoming backend connection from the proxy frontend is authenticated as the principal
-            the frontend proxy has _already_ authenticated the actual client (before even connecting and
-            authenticating to the backend here)
-            """
-            if isinstance(res, types.Deny):
+            if isinstance(res, Deny):
                 return res
 
-            principal = {}
-            principal['realm'] = details.authextra['proxy_realm']
-            principal['authid'] = details.authextra['proxy_authid']
-            principal['role'] = details.authextra['proxy_authrole']
-            principal['extra'] = details.authextra.get('proxy_authextra', None)
+            # the incoming backend connection from the proxy frontend is authenticated as the principal
+            # the frontend proxy has _already_ authenticated the actual client (before even connecting and
+            # authenticating to the backend here)
+            principal = {
+                'realm': details.authextra['proxy_realm'],
+                'authid': details.authextra['proxy_authid'],
+                'role': details.authextra['proxy_authrole'],
+
+                # the authextra intended for the principal is forwarded from the proxy
+                'extra': details.authextra.get('proxy_authextra', None)
+            }
             self._assign_principal(principal)
 
             self.log.debug(
@@ -377,8 +401,11 @@ class PendingAuthCryptosignProxy(PendingAuthCryptosign):
             )
             return self._accept()
 
-        def error(f):
-            return types.Deny("Internal error: {}".format(f))
+        def error(_err):
+            return Deny("Internal error: {}".format(_err))
 
         txaio.add_callbacks(f, assign, error)
         return f
+
+
+IPendingAuth.register(PendingAuthCryptosign)
