@@ -98,7 +98,7 @@ class Broker(object):
 
         # store for event history
         if self._router._store:
-            self._event_store = self._router._store.event_store
+            self._event_store = self._router._store
         else:
             self._event_store = None
 
@@ -343,7 +343,7 @@ class Broker(object):
             #
             if publish.payload is None:
                 try:
-                    self._router.validate('event', publish.topic, publish.args, publish.kwargs)
+                    self._router.validate('event', publish.topic, publish.args, publish.kwargs, validate=None)
                 except Exception as e:
                     if publish.acknowledge:
                         if self._router.is_traced:
