@@ -6,14 +6,26 @@
 #####################################################################################
 
 from collections import deque
+from typing import Optional, List, Dict, Any
 
+from txaio import use_twisted  # noqa
 from txaio import make_logger, time_ns
 
-__all__ = ('MemoryRealmStore', )
+from autobahn.util import hltype, hlval
+from autobahn.wamp.interfaces import ISession
+from autobahn.wamp.types import CloseDetails, SessionDetails
+from autobahn.wamp.message import Publish
+
+from crossbar.interfaces import IRealmStore
+from crossbar.router.observation import UriObservationMap
+
+__all__ = (
+    'RealmStoreMemory',
+    'QueuedCall',
+)
 
 
 class QueuedCall(object):
-
     __slots__ = ('session', 'call', 'registration', 'authorization')
 
     def __init__(self, session, call, registration, authorization):
@@ -23,18 +35,20 @@ class QueuedCall(object):
         self.authorization = authorization
 
 
-class MemoryCallQueue(object):
+class RealmStoreMemory(object):
     """
-    Memory-backed call queue.
+    Memory-backed realm store.
     """
 
     log = make_logger()
 
-    GLOBAL_QUEUE_LIMIT = 1000
+    STORE_TYPE = 'memory'
+
+    GLOBAL_HISTORY_LIMIT = 100
     """
-    The global call queue limit, in case not overridden.
+    The global history limit, in case not overridden.
     """
-    def __init__(self, config=None):
+    def __init__(self, personality, factory, config):
         """
 
         See the example here:
@@ -55,67 +69,15 @@ class MemoryCallQueue(object):
                 ]
             }
         """
-        # whole store configuration
-        self._config = config or {}
+        from twisted.internet import reactor
 
-        # limit to call queue per registration
-        self._limit = self._config.get('limit', self.GLOBAL_QUEUE_LIMIT)
+        self._reactor = reactor
+        self._personality = personality
+        self._factory = factory
+        self._config = config
 
-        # map: registration.id -> deque( (session, call, registration, authorization) )
-        self._queued_calls = {}
-
-    def maybe_queue_call(self, session, call, registration, authorization):
-        # FIXME: match this against the config, not just plain accept queueing!
-        if registration.id not in self._queued_calls:
-            self._queued_calls[registration.id] = deque()
-
-        self._queued_calls[registration.id].append(QueuedCall(session, call, registration, authorization))
-
-        return True
-
-    def get_queued_call(self, registration):
-        if registration.id in self._queued_calls and self._queued_calls[registration.id]:
-            return self._queued_calls[registration.id][0]
-
-    def pop_queued_call(self, registration):
-        if registration.id in self._queued_calls and self._queued_calls[registration.id]:
-            return self._queued_calls[registration.id].popleft()
-
-
-class MemoryEventStore(object):
-    """
-    Memory-backed event store.
-    """
-
-    log = make_logger()
-
-    STORE_TYPE = 'memory'
-
-    GLOBAL_HISTORY_LIMIT = 100
-    """
-    The global history limit, in case not overridden.
-    """
-    def __init__(self, config=None):
-        """
-
-        See the example here:
-
-        https://github.com/crossbario/crossbar-examples/tree/master/event-history
-
-        "store": {
-            "type": "memory",
-            "limit": 100,       // global default for history limit
-            "event-history": [
-                {
-                    "uri": "com.example.oncounter",
-                    "match": "exact",
-                    "limit": 1000       // topic specific history limit
-                }
-            ]
-        }
-        """
-        # whole store configuration
-        self._config = config or {}
+        self._type = self._config.get('type', None)
+        assert self._type == self.STORE_TYPE
 
         # limit to event history per subscription
         self._limit = self._config.get('limit', self.GLOBAL_HISTORY_LIMIT)
@@ -129,7 +91,78 @@ class MemoryEventStore(object):
         # map of subscription ID -> (limit, deque(of publication IDs))
         self._event_history = {}
 
-    def attach_subscription_map(self, subscription_map):
+        # map: registration.id -> deque( (session, call, registration, authorization) )
+        self._queued_calls = {}
+
+        self._running = False
+
+        self.log.info('{func} realm store initialized (type="{stype}")',
+                      stype=hlval(self._type),
+                      func=hltype(self.__init__))
+
+    def type(self) -> str:
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.type`
+        """
+        return self._type
+
+    def is_running(self) -> bool:
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.is_running`
+        """
+        return self._running
+
+    def start(self):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.start`
+        """
+        if self._running:
+            raise RuntimeError('store is already running')
+        else:
+            self.log.info(
+                '{func} starting realm store type="{stype}"',
+                func=hltype(self.start),
+                stype=hlval(self._type),
+            )
+
+        # currently nothing to do in stores of type "memory"
+        self._running = True
+        self.log.info('{func} realm store ready!', func=hltype(self.start))
+
+    def stop(self):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.stop`
+        """
+        if not self._running:
+            raise RuntimeError('store is not running')
+        else:
+            self.log.info('{func} stopping realm store', func=hltype(self.start))
+
+        # currently nothing to do in stores of type "memory"
+        self._running = False
+
+    def store_session_joined(self, session: ISession, details: SessionDetails):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.store_session_joined`
+        """
+        self.log.info('{func} new session joined session={session}, details={details}',
+                      func=hltype(self.store_session_joined),
+                      session=session,
+                      details=details)
+
+    def store_session_left(self, session: ISession, details: CloseDetails):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.store_session_left`
+        """
+        self.log.info('{func} session left session={session}, details={details}',
+                      func=hltype(self.store_session_left),
+                      session=session,
+                      details=details)
+
+    def attach_subscription_map(self, subscription_map: UriObservationMap):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.attach_subscription_map`
+        """
         for sub in self._config.get('event-history', []):
             uri = sub['uri']
             match = sub.get('match', 'exact')
@@ -141,32 +174,9 @@ class MemoryEventStore(object):
             # for in-memory history, we just use a double-ended queue
             self._event_history[subscription_id] = (sub.get('limit', self._limit), deque())
 
-    def store_session_joined(self, session, session_details):
-        self.log.debug('{klass}.store_session_join(session={session}, session_details={session_details})',
-                       klass=self.__class__.__name__,
-                       session=session,
-                       session_details=session_details)
-
-    def store_session_left(self, session, session_details, close_details):
-        self.log.debug(
-            '{klass}.store_session_left(session={session}, session_details={session_details}, close_details={close_details})',
-            klass=self.__class__.__name__,
-            session=session,
-            session_details=session_details,
-            close_details=close_details)
-
-    def store_event(self, session, publication_id, publish):
+    def store_event(self, session: ISession, publication_id: int, publish: Publish):
         """
-        Store event to event history.
-
-        :param session: The publishing session.
-        :type session: :class:`autobahn.wamp.interfaces.ISession`
-
-        :param publication_id: The WAMP publication ID under which the publish happens
-        :type publication_id: int
-
-        :param publish: The WAMP publish message.
-        :type publish: :class:`autobahn.wamp.messages.Publish`
+        Implements :meth:`crossbar._interfaces.IRealmStore.store_event`
         """
         assert (publication_id not in self._event_store)
         evt = {
@@ -185,16 +195,9 @@ class MemoryEventStore(object):
                        store_type=self.STORE_TYPE,
                        publication_id=publication_id)
 
-    def store_event_history(self, publication_id, subscription_id, receiver):
+    def store_event_history(self, publication_id: int, subscription_id: int, receiver: ISession):
         """
-        Store publication history for subscription.
-
-        :param publication_id: The ID of the event publication to be persisted.
-        :type publication_id: int
-
-        :param subscription_id: The ID of the subscription the event (identified by the publication ID),
-            was published to, because the event's topic matched the subscription.
-        :type subscription_id: int
+        Implements :meth:`crossbar._interfaces.IRealmStore.store_event_history`
         """
         # assert(publication_id in self._event_store)
         # assert(subscription_id in self._event_history)
@@ -245,19 +248,9 @@ class MemoryEventStore(object):
                 del self._event_store[purged_publication_id]
                 self.log.debug("Event {publication_id} purged completey", publication_id=purged_publication_id)
 
-    def get_events(self, subscription_id, limit):
+    def get_events(self, subscription_id: int, limit: Optional[int] = None):
         """
-        Retrieve given number of last events for a given subscription.
-
-        If no history is maintained for the given subscription, None is returned.
-
-        :param subscription_id: The ID of the subscription to retrieve events for.
-        :type subscription_id: int
-        :param limit: Limit number of events returned.
-        :type limit: int
-
-        :return: List of events.
-        :rtype: list or None
+        Implements :meth:`crossbar._interfaces.IRealmStore.get_events`
         """
         if subscription_id not in self._event_history:
             return None
@@ -267,7 +260,7 @@ class MemoryEventStore(object):
             # at most "limit" events in reverse chronological order
             res = []
             i = -1
-            if limit > len(history):
+            if limit is None or limit > len(history):
                 limit = len(history)
             for _ in range(limit):
                 publication_id = history[i]
@@ -275,43 +268,37 @@ class MemoryEventStore(object):
                 i -= 1
             return res
 
-    def get_event_history(self, subscription_id, from_ts, until_ts):
+    def get_event_history(self, subscription_id: int, from_ts: int, until_ts: int) -> Optional[List[Dict[str, Any]]]:
         """
-        Retrieve event history for time range for a given subscription.
-
-        If no history is maintained for the given subscription, None is returned.
-
-        :param subscription_id: The ID of the subscription to retrieve events for.
-        :type subscription_id: int
-        :param from_ts: Filter events from this date (string in ISO-8601 format).
-        :type from_ts: unicode
-        :param until_ts: Filter events until this date (string in ISO-8601 format).
-        :type until_ts: unicode
+        Implements :meth:`crossbar._interfaces.IRealmStore.get_event_history`
         """
         raise Exception("not implemented")
 
-
-class MemoryRealmStore(object):
-    """
-    Memory backed realm store.
-    """
-
-    log = make_logger()
-
-    event_store = None
-    """
-    Store for event history.
-    """
-
-    call_store = None
-    """
-    Store for call queueing.
-    """
-    def __init__(self, personality, factory, config):
+    def maybe_queue_call(self, session, call, registration, authorization):
         """
-
-        :param config: Realm store configuration item.
-        :type config: Mapping
+        Implements :meth:`crossbar._interfaces.IRealmStore.maybe_queue_call`
         """
-        self.event_store = MemoryEventStore(config)
-        self.call_store = MemoryCallQueue(config)
+        # FIXME: match this against the config, not just plain accept queueing!
+        if registration.id not in self._queued_calls:
+            self._queued_calls[registration.id] = deque()
+
+        self._queued_calls[registration.id].append(QueuedCall(session, call, registration, authorization))
+
+        return True
+
+    def get_queued_call(self, registration):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.get_queued_call`
+        """
+        if registration.id in self._queued_calls and self._queued_calls[registration.id]:
+            return self._queued_calls[registration.id][0]
+
+    def pop_queued_call(self, registration):
+        """
+        Implements :meth:`crossbar._interfaces.IRealmStore.pop_queued_call`
+        """
+        if registration.id in self._queued_calls and self._queued_calls[registration.id]:
+            return self._queued_calls[registration.id].popleft()
+
+
+IRealmStore.register(RealmStoreMemory)
