@@ -305,7 +305,7 @@ class RouterRoleDynamicAuth(RouterRole):
         :type router: instance of ``crossbar.router.router.Router``
         :param id: The URI of the role.
         :type id: unicode
-        :param authorizer: The dynamic authroizer configuration.
+        :param authorizer: The dynamic authorizer configuration.
         :type authorizer: dict
         """
         RouterRole.__init__(self, router, uri)
@@ -335,6 +335,8 @@ class RouterRoleDynamicAuth(RouterRole):
         :type uri: str
         :param action: The action to be performed.
         :type action: str
+        :param options:
+        :type options:
 
         :return: bool -- Flag indicating whether session is authorized or not.
         """
@@ -355,6 +357,7 @@ class RouterRoleDynamicAuth(RouterRole):
                 }
             }
         else:
+            _td = session._transport.transport_details.marshal() if session._transport.transport_details else None
             details = {
                 'session': session_details.session,
                 'authid': session_details.authid,
@@ -362,7 +365,7 @@ class RouterRoleDynamicAuth(RouterRole):
                 'authmethod': session_details.authmethod,
                 'authprovider': session_details.authprovider,
                 'authextra': session_details.authextra,
-                'transport': session._transport._transport_info
+                'transport': _td
             }
 
         self.log.debug("CrossbarRouterRoleDynamicAuth.authorize {uri} {action} {details}",
@@ -393,16 +396,26 @@ class RouterRoleDynamicAuth(RouterRole):
             Ensure the return-value we got from the user-supplied method makes sense
             """
             if isinstance(authorization, dict):
+                # check keys
                 for key in authorization.keys():
-                    if key not in ['allow', 'cache', 'disclose']:
+                    if key not in ['allow', 'cache', 'disclose', 'validate', 'meta']:
                         return Failure(ValueError("Authorizer returned unknown key '{key}'".format(key=key, )))
-                # must have "allow"
+                # must have "allow" key
                 if 'allow' not in authorization:
                     return Failure(ValueError("Authorizer must have 'allow' in returned dict"))
-                # all values must be bools
-                for key, value in authorization.items():
-                    if not isinstance(value, bool):
-                        return Failure(ValueError("Authorizer must have bool for '{}'".format(key)))
+                # check bool-valued keys
+                for key in ['allow', 'cache', 'disclose']:
+                    if key in authorization:
+                        value = authorization[key]
+                        if not isinstance(value, bool):
+                            return Failure(ValueError("Authorizer must have bool for '{}'".format(key)))
+                # check dict-valued keys
+                for key in ['validate', 'meta']:
+                    if key in authorization:
+                        value = authorization[key]
+                        if value is not None and not isinstance(value, dict):
+                            return Failure(
+                                ValueError("Authorizer must have dict for '{}' (if present and not null)".format(key)))
                 return authorization
 
             elif isinstance(authorization, bool):
