@@ -154,33 +154,51 @@ class Application(object):
         self._output_style = 'fruity'
 
     @staticmethod
-    def load_profile(dotdir=None, profile=None, yes_to_all=False):
+    def load_profile(dotdir=None, profile=None, yes_to_all=False, verbose=False):
 
-        dotdir = dotdir or '~/.crossbar'
         profile = profile or 'default'
 
-        cbf_dir = os.path.expanduser(dotdir)
+        if not dotdir:
+            if 'CROSSBAR_FABRIC_SUPERUSER' in os.environ:
+                cbf_dir = os.path.abspath(os.path.dirname(os.environ['CROSSBAR_FABRIC_SUPERUSER']))
+                if verbose:
+                    click.echo('Using dotdir derived from CROSSBAR_FABRIC_SUPERUSER: {}'.format(style_ok(cbf_dir)))
+            else:
+                cbf_dir = os.path.abspath(os.path.expanduser('~/.crossbar'))
+                if verbose:
+                    click.echo('Using default dotdir: {}'.format(style_ok(cbf_dir)))
+        else:
+            cbf_dir = os.path.abspath(os.path.expanduser(dotdir))
+            if verbose:
+                click.echo('Using explicit dotdir: {}'.format(style_ok(cbf_dir)))
+
         if not os.path.isdir(cbf_dir):
             os.mkdir(cbf_dir)
-            click.echo('Created new local user directory: {}'.format(style_ok(cbf_dir)))
+            if verbose:
+                click.echo('Created new local user directory: {}'.format(style_ok(cbf_dir)))
 
         config_path = os.path.join(cbf_dir, 'config.ini')
         if not os.path.isfile(config_path):
             with open(config_path, 'w') as f:
                 url = _prompt_for_url(yes_to_all)
                 f.write(_DEFAULT_CONFIG.format(url=url))
-                click.echo('Created new local user configuration: {}'.format(style_ok(config_path)))
+                if verbose:
+                    click.echo('Created new local user configuration: {}'.format(style_ok(config_path)))
+        else:
+            if verbose:
+                click.echo('Using existing local user configuration: {}'.format(style_ok(config_path)))
 
         config_obj = config.UserConfig(config_path)
 
         profile_obj = config_obj.profiles.get(profile, None)
         if not profile_obj:
             raise click.ClickException('no such profile: "{}"'.format(profile))
-        # else:
-        #     click.echo('Active user profile: {}'.format(style_ok(profile)))
+        else:
+            if verbose:
+                click.echo('Active user profile: {}'.format(style_ok(profile)))
 
         privkey_path = os.path.join(cbf_dir, profile_obj.privkey or '{}.priv'.format(profile))  # noqa: W503
-        pubkey_path = os.path.join(cbf_dir, profile_obj.pubkey or 'default.pub')  # noqa: W503
+        pubkey_path = os.path.join(cbf_dir, profile_obj.pubkey or '{}.pub'.format(profile))  # noqa: W503
         key_obj = userkey.UserKey(privkey_path, pubkey_path, yes_to_all=yes_to_all)
 
         return key_obj, profile_obj
@@ -252,7 +270,11 @@ class Application(object):
         cmd_str = ' '.join(["crossbar", "shell"] + sys.argv[1:])
         if self._output_format in [Application.OUTPUT_FORMAT_JSON, Application.OUTPUT_FORMAT_JSON_COLORED]:
 
-            json_str = json.dumps(result.result, separators=(', ', ': '), sort_keys=True, indent=4, ensure_ascii=False)
+            json_str = json.dumps(result.result,
+                                  separators=(', ', ': '),
+                                  sort_keys=False,
+                                  indent=4,
+                                  ensure_ascii=False)
 
             if self._output_format == Application.OUTPUT_FORMAT_JSON_COLORED:
                 console_str = highlight(json_str, lexers.JsonLexer(),
@@ -337,7 +359,10 @@ class Application(object):
             click.echo('Crossbar.io Shell: {}'.format(style_ok('v{}'.format(__version__))))
 
         # load user profile and key for given profile name
-        key, profile = self.load_profile(profile=cfg.profile, yes_to_all=yes_to_all)
+        key, profile = self.load_profile(dotdir=cfg.dotdir,
+                                         profile=cfg.profile,
+                                         yes_to_all=yes_to_all,
+                                         verbose=(ctx.command.name == 'init'))
 
         if ctx.command.name == 'init':
             return
@@ -364,6 +389,8 @@ class Application(object):
         # realm, or a management realm the user has a role on)
         done = txaio.create_future()
 
+        url_is_secure, _, _, _, _, _ = parse_url(url)
+
         extra = {
             # these are forward on the actual client connection
             'authid': authid,
@@ -372,7 +399,10 @@ class Application(object):
             # these are native Python object and only used client-side
             'key': key.key,
             'done': done,
-            'command': command
+            'command': command,
+
+            # WAMP-cryptosign authentication: TLS channel binding
+            'channel_binding': 'tls-unique' if url_is_secure else None,
         }
 
         cert_options = None
