@@ -8,13 +8,17 @@
 import os
 import sys
 import pkg_resources
-import jinja2
 import signal
+from typing import Optional, List
+
+import jinja2
+from jinja2.sandbox import SandboxedEnvironment
+from jinja2 import Environment
 
 from twisted.internet.error import ReactorNotRunning
 from twisted.internet.defer import inlineCallbacks
 
-from autobahn.util import utcnow
+from autobahn.util import utcnow, hltype
 from autobahn.wamp.exception import ApplicationError
 from autobahn.wamp.types import PublishOptions, Challenge
 from autobahn import wamp
@@ -63,19 +67,38 @@ class WorkerController(NativeProcess):
 
         # Jinja2 templates for Web (like WS status page et al)
         #
-        template_dirs = []
+        self._templates_dir = []
         for package, directory in self.personality.TEMPLATE_DIRS:
             dir_path = os.path.abspath(pkg_resources.resource_filename(package, directory))
-            template_dirs.append(dir_path)
-        self.log.debug("Using Web templates from {template_dirs}", template_dirs=template_dirs)
-        self._templates = jinja2.Environment(loader=jinja2.FileSystemLoader(template_dirs), autoescape=True)
+            self._templates_dir.append(dir_path)
+        self.log.debug("Using Web templates from {template_dirs}", template_dirs=self._templates_dir)
+
+        # FIXME: make configurable, but default should remain SandboxedEnvironment for security
+        if True:
+            # The sandboxed environment. It works like the regular environment but tells the compiler to
+            # generate sandboxed code.
+            # https://jinja.palletsprojects.com/en/2.11.x/sandbox/#jinja2.sandbox.SandboxedEnvironment
+            self._templates = SandboxedEnvironment(loader=jinja2.FileSystemLoader(self._templates_dir),
+                                                   autoescape=True)
+        else:
+            self._templates = Environment(loader=jinja2.FileSystemLoader(self._templates_dir), autoescape=True)
 
         self.join(self.config.realm)
 
-    def templates(self):
+    @property
+    def templates_dir(self) -> List[str]:
         """
+        Template directories used in the Jinja2 rendering environment.
 
-        :return: jinja2.Environment for the built in templates from personality.TEMPLATE_DIRS
+        :return:
+        """
+        return self._templates_dir
+
+    def templates(self) -> Environment:
+        """
+        Jinja2 rendering environment.
+
+        :return: jinja2.Environment for the built-in templates from personality.TEMPLATE_DIRS
         """
         return self._templates
 
@@ -360,11 +383,29 @@ class WorkerController(NativeProcess):
         return res
 
     @inlineCallbacks
-    def sign_challenge(self, challenge: Challenge, channel_id):
-        result = yield self.call("crossbar.sign_challenge", challenge.method, challenge.extra, channel_id)
+    def sign_challenge(self, challenge: Challenge, channel_id: Optional[bytes], channel_id_type=Optional[str]):
+        """
+        Call into node controller (over secure controller-worker pipe) to sign challenge with node key.
+
+        :param challenge:
+        :param channel_id:
+        :param channel_id_type:
+        :return:
+        """
+        self.log.info('{func}() ...', func=hltype(self.sign_challenge))
+        result = yield self.call("crossbar.sign_challenge", challenge.method, challenge.extra, channel_id,
+                                 channel_id_type)
+        self.log.info('{func}(): {result}', func=hltype(self.sign_challenge), result=result)
         return result
 
     @inlineCallbacks
     def get_public_key(self):
+        """
+        Call into node controller (over secure controller-worker pipe) to get the node's public key.
+
+        :return:
+        """
+        self.log.info('{func}() ...', func=hltype(self.get_public_key))
         result = yield self.call("crossbar.get_public_key")
+        self.log.info('{func}(): {result}', func=hltype(self.get_public_key), result=result)
         return result
